/* SGMLFilter.c -- Copyright 1993 Liam R. Quin.  All Rights Reserved.
 * This code is NOT in the public domain.
 * See the file COPYRIGHT for full details.
 */

/* $Id: SGMLFilter.c,v 1.2 93/12/08 23:21:45 lee Exp $
 */

/* Filter for ISO 8879 (SGML) files
 *
 */

#ifdef SYSV
 extern int _filbuf(), _flsbuf(); /* for lint! */
#endif

#include "globals.h"
#include "error.h"

#include <stdio.h>
#include <malloc.h>
#include <ctype.h>
#include "wordrules.h"

#include "emalloc.h"

#ifndef STREQ
# define STREQ(boy, girl) ((*(boy) == *(girl)) && !strcmp(boy, girl))
#endif

#ifndef START_ONLY_ONCE
# define START_ONLY_ONCE { static char _x = 0; if (!_x) { _x = 1;
# define END_ONLY_ONCE } }
#endif

/** C Library functions that need to be declared: **/
#ifndef tolower
 extern int tolower();
#endif
extern int strcmp();
/** Functions in this file that need to be declared **/
INLINE static int GetChar();

/** **/

extern char *progname;

void Filter();

static int LastChar = 0;
static int InWord = 0;
static int LastInWord = 0;

INLINE static void
IgnoreChar(ch)
    int ch;
{
    switch (InWord) {
    case 0:
	if (isspace(ch) || isdigit(ch) || ispunct(ch)) {
	    putchar(ch);
	} else {
	    putchar(' ');
	}
	break;
    case 1:
	putchar('q');
	break;
    default:
	putchar('x');
	break;
    }
}

INLINE static int
GetChar(fd)
    FILE *fd;
{
    if (LastChar) {
	int ch = LastChar;
	InWord = LastInWord;
	LastChar = 0;
	return ch;
    }

    if ((LastChar = getc(fd)) == EOF) return EOF;

    LastInWord = InWord;

    if (InWord == 0) {
	InWord = StartsWord(LastChar);
    } else {
	if (EndsWord(LastChar) || OnlyWithinWord(LastChar)) {
	    ++InWord;
	} else {
	    InWord = 0;
	}
    }

    /* Only return a single quote if it is within a word:
     * can't --- OK
     * ...hello' he said --- rejected
     * '' --- rejected
     * 30's --- OK
     * 30'66" -- rejected
     */
    if (LastChar == '\'') {
	LastChar = getc(fd);
	if (InWord && isalpha(LastChar)) {
	    return '\'';
	} else {
	    InWord = 0;
	    return '"';
	}
    } else {
	int ch = LastChar;
	LastChar = 0;
	return ch;
    }
}

INLINE static int
UnGetChar(fd, ch)
    FILE *fd;
    int ch;
{
    if (LastChar) {
	(void) ungetc(LastChar, stdin);
    }
    LastChar = ch;
}

#define issgmldelim(ch) (isspace(ch) || ch == ';' || ch == '<')

void
Filter(InputFile, Name)
    FILE *InputFile;
    char *Name;
{
    int WithinATag = 0;
    int WithinAString = 0;
    int WithinSpecial = 0;
    char QuoteChar = 0;
    int ch;

    while ((ch = GetChar(InputFile)) != EOF) {
	if (ch == '<' && !WithinATag) {
	    WithinATag = 1;
	    putchar(' ');
	    switch (ch = GetChar(InputFile)) {
	    case EOF:
		fflush(stdout);
		Error(E_FATAL, "%s: End of file within a tag", Name);
	    case '!': /* <! ..... > */
		WithinSpecial = 1;
		putchar(' ');
		continue;
	    case '\n':
	    case ' ':
	    case '\t':
	    case '\r':
		putchar(ch);
		continue;
	    case '>':
		Error(E_FATAL, "%s: Badly formed SGML, found <>", Name);
	    default:
		IgnoreChar(ch);
		continue;
	    }
	}
	if (WithinATag) {
	    /* special processing for attributes, ! and so forth */
	    if (WithinAString) {
		if (ch == QuoteChar) {
		    WithinAString = 0;
		    QuoteChar = 0;
		}
		IgnoreChar(ch);
		continue;
	    } else {
		/* within a tag but not a string */
		while (isspace(ch)) {
		    IgnoreChar(ch);
		    ch = GetChar(InputFile);
		}
		switch (ch) {
		case '"': case '\'':
		    WithinAString = 1;
		    QuoteChar = ch;
		    break;
		case '<': /* for DOCTYPE etc... */
		    if (WithinSpecial) {
			WithinATag++;
		    } else {
			START_ONLY_ONCE {
			    Error(E_WARN, "%s: < within a tag!", Name);
			} END_ONLY_ONCE;
		    }
		    break;
		case '>':
		    WithinATag--;
		    if (WithinSpecial && WithinATag == 0) {
			WithinSpecial = 0;
		    }
		    break;
		case '-': /* TODO NOTDONE FIXME do SGML comments */
		default:
		    break;
		}
		IgnoreChar(ch);
		continue;
	    }
	} else { /* not within a tag */
	    /* TODO: handle entities
	     * this is tricky to get right whilst mantaining the byte count...
	     * variable sized *input* blocks required for this.
	     */
	    if (ch == '<') {
		WithinATag++;
	    } else if (ch == '&') {
		do {
		    IgnoreChar(ch);
		    ch = GetChar(InputFile);
		    if (ch == EOF) {
			Error(E_WARN,
			    "%s: end of file inside an entity!",
			    Name
			);
		    }
		} while (!issgmldelim(ch));
		if (ch == ';') {
		    putchar(' ');
		} else {
		    UnGetChar(InputFile, ch);
		    continue;
		}
	    } else {
		putchar(ch);
	    }
	}
    }
}


void
Body(InputFile, Name)
    FILE *InputFile;
    char *Name;
{
    int ch;
    register char *p;
    char *Line;
    int LineState = 0;
    int CheckForShar = 0; /* seen a line starting w/ "-" "#!" or ":" recently */
    char *EOFStr = 0;

    return;
}
