/* readfile.h -- Copyright 1992 Liam R. Quin.  All Rights Reserved.
 * This code is NOT in the public domain.
 * See the file COPYRIGHT for full details.
 */
 
/* $Id: readfile.h,v 1.3 93/07/19 17:22:16 lee Exp $ */
 
/* Flags for fReadFile */

#ifndef READFILE_H
# define UF_IGNBLANKS	001 /* throw away blank lines */
# define UF_IGNSPACES	002 /* discard leading and trailing spaces */
# define UF_IGNHASH  	004 /* discard LEADING comments (# with a hash-sign) */
# define UF_ESCAPEOK 	010 /* accept \# and \\ as # and \ */
# define UF_IGNALLHASH   020 /* discard ALL comments (# with a hash-sign) */
# define UF_NORMAL (UF_IGNBLANKS|UF_IGNSPACES|UF_IGNHASH|UF_ESCAPEOK)


/** File and Line Reading Routines:
 **
 **/

extern long
ReadFile(/* int Severity; char *Name, *What; char ***Lines; int Flags */);
/**	Reads lines from the named file (Name) and puts them in an array
 **	that it allocates with malloc().  Returns a pointer to the
 **	start of the array of pointers to lines (in *Lines), and
 **	the number of lines read.  If the file can't be opened, or it runs
 **	out of memory, ReadFile() calls Error() with the given Severity,
 **	and with an error message constructed out of What, which should
 **	be a short (e.g. 3-word) description of the purpose of the file.
 **	The Flags can currently include
 **		UF_IGNBLANKS to throw away blank lines,
 **		UF_IGNSPACES to discard leading and trailing spaces,
 **		UF_IGNHASH   to discard leading comments (# with a hash-sign)
 **		UF_IGNALLHASH   to discard comments (# with a hash-sign)
 **		UF_ESCAPEOK  to accept \# and \\ as # and \
 **	this file also defines UF_NORMAL, see above.
 **/

extern long
fReadFile(/* int S; FILE *f; char *Name, *What; char ***Lines; int Flags */);
/**	is identical except that it takes an already-opened FILE pointer and
 **	reads up to EOF.
 **/

extern int
fReadLine(/* FILE *f, char **Linep; int Flags */);
/**	This reads a single line into memory, making room with malloc()
 **	and setting *Linep to point to the new storage.  It returns -1
 **	when it detects the end of input (EOF).
 **	If malloc() or realloc returns 0 (indicating that no more memory
 **	is available), *Linep is set to 0 and the number of characters
 **	read is returned.
 **	The Flags argument is the same as for ReadFile.
 **/
#endif
