/* readline.c -- Copyright 1993 Liam R. E. Quin.  All Rights Reserved.
 * This code is NOT in the public domain.
 * See the file COPYRIGHT for full details.
 */

/* $Id: readfile.c,v 1.6 92/07/05 12:50:06 lee Exp $
 */

#include <stdio.h>
#include <ctype.h>

#include "globals.h"
#include "emalloc.h"

extern int AsciiTrace;

#include <stdio.h>
#include <ctype.h>	/* isspace() etc */
#include <malloc.h>	/* This declares malloc(), realloc() etc... */

#include "globals.h"
#include "error.h"
#include "emalloc.h"

#include "readfile.h"

int
fReadLine(f, Linep, Flags)
    FILE *f;
    char **Linep;
    int Flags;
{
    register char *p;
    static int LinesRead = 0;
    unsigned int LineLength = 0;
    unsigned long TotalSoFar = 0L;
    int c;

    /* NOTE: we mantain a running average of linelengths, and use
     * this as a heuristic for the initial buffer size.  At the end,
     * we realloc() if we over-estimated by more than one byte.
     */

TryAgain:

    *Linep = 0;
    p = (*Linep);

    while ((c = getc(f)) != EOF && c != '\n') {
	/* Look for things to skip before allocating any space: */

	if (c == '\\' && (Flags & UF_ESCAPEOK)) {
	    if ((c = getc(f)) == EOF) break;
	    else if (c == '\n') continue; /* join lines together */
	} else if (c == '#' && ((Flags & UF_IGNALLHASH) ||
				((Flags & UF_IGNHASH) && p == (*Linep)))) {
	    while ((c = getc(f)) != EOF && c != '\n') {
		/*NULLBODY*/ ;
	    }
	    if (c == EOF) {
		break;
	    } else {
		(void) ungetc(c, f);
		continue;
	    }
	} else if (p == (*Linep) && isspace(c) && (Flags & UF_IGNSPACES)) {
	    continue; /* ignore leading blanks */
	}

	/* Now ensure that we have allocated enpough space: */
	if (!p || p - (*Linep) >= LineLength - 1) {
	    /* the -1 is to leave room for a trailing \0 */

	    int WhereWeWere = p - (*Linep);

	    if (LineLength == 0) {
		LineLength = TotalSoFar / (LinesRead ? LinesRead : 1);
		if (LineLength < 2) {
		    LineLength = 10;
			/* it seems silly allocating less than this... */
		    LinesRead = 0;
		}
		*Linep = malloc(LineLength);
	    } else {
		if (LineLength < 20) LineLength += 10;
		else LineLength += (LineLength / 2);

		*Linep = realloc(*Linep, LineLength);
	    }

	    if (!*Linep) return WhereWeWere + 1;

	    p = (*Linep);
	    p = &p[WhereWeWere];
	}
	*p++ = c;
	*p = '\0';  /* p a r a n o i a */
    }

    if (!p) {
	if (c == EOF) {
	    return -1;
	} else if (Flags & UF_IGNBLANKS) {
	    if (*Linep) (void) free(*Linep);
	    *Linep = (char *) 0;
	    goto TryAgain;
	} else {
	    *Linep = (char *) 0;
	    return -1;
	}
    }

    *p = '\0';

    /* Discard a trailing \r, as these are usually the result of
     * file transfers (or using script(1) on 4.2 BSD Unix) and are
     * unwanted.
     * I hope.  (is this a bogus mis-feature?  I don't think so, but then,
     * I wrote it...)
     */
    if (p > *Linep && p[-1] == '\r') {
	*--p = '\0';
    }

    if (Flags & UF_IGNSPACES) {
	/* delete trailing spaces, being careful to keep "\ " if requested */
	while (p > *Linep && isspace(p[-1])) {
	    if (&p[-1] > *Linep && p[-2] == '\\' && (Flags && UF_ESCAPEOK)) {
		break; /* This is the case where we must keep "\ " */
	    }
	    *--p = '\0';
	}
    }

    TotalSoFar += p - *Linep;
    ++LinesRead;
	/* Note: we include blank lines in the average */

    if (p == *Linep) {
	if (c == EOF) {
	    return -1;
	} else if (Flags & UF_IGNBLANKS) {
	    (void) free(*Linep);
	    *Linep = (char *) 0;	/* p a r a n o i a */
	    goto TryAgain;
	 }
    }

    /* ASSERT: p > *Linep */

    if (p - *Linep < LineLength - 2) {
	/* The +2 in the test above is one for a \0 and one extra to reduce
	 * the number of relloc() calls -- an extra byte per line is no
	 * disaster, since we discard the \n anyway.
	 */
	LineLength = p - *Linep;
	    /* ASSERT: LineLength != 0 */
	if ((*Linep = realloc(*Linep, LineLength + 1)) == (char *) 0) {
	    return LineLength;
	}

	p = (*Linep) + LineLength;

    }
    return p - *Linep;
}
